import asyncio

from quart import Quart, websocket

from glebby.model import Model

model = Model()
app = Quart(__name__, static_url_path='')

async def send_outgoing(queue):
    while True:
        data = await queue.get()
        await websocket.send(data)

@app.route('/')
async def root():
    return await app.send_static_file('index.html')

@app.websocket('/glebby')
async def ws():
    global model

    # For each connected client, we create a queue to store its outgoing messages.
    # We pass this queue to the model so that it can communicate with the clients.
    outgoing_queue = asyncio.Queue()
    client_id = await model.add_client(outgoing_queue)

    try:
        # In order to get the queued messages through the websocket, we need to create a task
        # since the current one will be busy receiving incoming messages below.  
        outgoing_task = asyncio.create_task(send_outgoing(outgoing_queue))

        while True:
            data = await websocket.receive()
            await model.handle_incoming(client_id, data)
    # When a client disconnects, we shut down the message pumping task and wait for it to finish
    except asyncio.CancelledError:
        await model.remove_client(client_id)
        outgoing_task.cancel()
        await outgoing_task
